
#include "SPI.h"
#include "Adafruit_GFX.h"
#include "Adafruit_ILI9341.h"
#include <XPT2046_Touchscreen.h>
#include "max6675.h"
#include "pid.h"

// Configuración de pines (Hardrware SPI)
#define TFT_DC 9
#define TFT_CS 10
#define TOUCH_CS 7
#define TFT_RST 8


int thermoDO = 4;
int thermoCS = 3;
int thermoCLK = 2;

MAX6675 thermocouple(thermoCLK, thermoCS, thermoDO);

#define HEATER_PIN 6
#define FAN_PIN 5
// Define the temperature profile and time profile
int tempProfile[] = {21, 150, 183, 217, 240, 240, 217, 183, 150, 100, 60};
int timeProfile[] = {1, 30, 60, 90, 120, 150, 180, 210, 240, 270, 300};
double currentTime = 1.0;
double dt = 0.1;

// Define the target temperature and PID constants
double targetTemperature = 30;
#define Kp 0.01
#define Ki 0.00001
#define Kd 100.0


// Creo el controlador pid
Pid control(Kp,Ki,Kd);


// Create the MAX6675 object

// pantalla
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);
XPT2046_Touchscreen ts(TOUCH_CS);

void setup() {
  //TCCR0B=TCCR0B & B11111000 | B00000100;//PRESCALER CANVIAR FREQUENCIA PER TREURE EL SOROLL
    //reset pantalla
  pinMode(TFT_RST, OUTPUT);
  digitalWrite(TFT_RST, LOW);
  delay(1000);
  digitalWrite(TFT_RST, HIGH);
  delay(1000);
    // Initialize the serial communication 
  Serial.begin(9600);
  //init screen
  tft.begin(1000000);
  tft.setRotation(3);
  //init Touch
  ts.begin();
  ts.setRotation(3);
  
  // Set the heater and fan pins to outputs
  pinMode(HEATER_PIN, OUTPUT);
  pinMode(FAN_PIN, OUTPUT);

  // Turn on the fan
  digitalWrite(FAN_PIN, LOW);
  //DRAW ALL THE GRAPHICS IN THE SCREEN
  
  dibujaGraficos();
//nO TINDRE EN COMPTE EL PRIMER TOUCH QUE FA
////////////  
  while (!ts.touched()) {
  }
  TS_Point p = ts.getPoint();
  //Serial.print("p.x: "); Serial.print(p.x); Serial.print(" p.y: "); Serial.println(p.y);
  delay(2000);
}
unsigned long lastFrame = millis();
int state = 0;
void loop() {
  if (!ts.touched()) {
    //Serial.println(state);
    tft.fillRect(210, 150, 110, 30, ILI9341_BLACK);
    tft.setTextColor(ILI9341_WHITE);
    tft.setCursor(210, 155); tft.setTextSize(3);
    tft.println(thermocouple.readCelsius());
    
    
    
    //Serial.println(thermocouple.readCelsius());
    if (state == 1) {
      double temperature = thermocouple.readCelsius();
//////////////
      currentTime += dt;
      int profileIndex = 1;
      int profileIndexANT =0;
      while (currentTime > timeProfile[profileIndex]) {
        profileIndexANT=profileIndex;
        profileIndex++;
      }
      //targetTemperature=tempProfile[profileIndex];
      targetTemperature=((tempProfile[profileIndex]-tempProfile[profileIndexANT])/(timeProfile[profileIndex]-timeProfile[profileIndexANT]))*(currentTime-timeProfile[profileIndexANT])+ tempProfile[profileIndexANT];
////////////////////
  // Read the temperature from the MAX6675 amplifier
      control.set_setpt(targetTemperature);
      control.set_input(temperature);

      int output = control.get_output();
      output = output>255? 255 : output;
      output = output<0? 0 : output;

      analogWrite(HEATER_PIN, output);

  // Update the heater and fan based on the PID output
      if (control.get_error() < -5) {
        digitalWrite(FAN_PIN, HIGH);
        //Serial.println("on");
        
      } else {
        digitalWrite(FAN_PIN, LOW);
        //Serial.println("off");
      }

      // Print the temperature and PID output to the serial monitor
      //Serial.print("Temperature: ");
      Serial.println(temperature);
      Serial.print(",");
      Serial.print(targetTemperature);
      Serial.print(",");
      Serial.println(output);
      //Serial.print("num");
      //Serial.println(profileIndex);
      //Serial.println(profileIndexANT);
      //Serial.print("temps");
      //Serial.println(currentTime);
      //Serial.println(state);
      //Serial.print(", Output: ");
      //Serial.println(output);
      //Serial.print(", error: ");
      //Serial.println(error);
      //Serial.println(tempProfile[profileIndex]-tempProfile[profileIndexANT]);
      //Serial.println(timeProfile[profileIndex]-timeProfile[profileIndexANT]);
      //Serial.println(currentTime-timeProfile[profileIndexANT]);
      tft.fillRect(210, 80, 110, 30, ILI9341_BLACK);
      tft.setTextColor(ILI9341_WHITE);
      tft.setCursor(210, 90); tft.setTextSize(3);
      tft.println(targetTemperature);
      if (profileIndex>=10){
        state=0;
        profileIndex=1;
        verd();
      }
    }
  }
  else {
    TS_Point p = ts.getPoint();
    //Serial.print("p.x: "); Serial.print(p.x); Serial.print(" p.y: "); Serial.println(p.y);
    //delay(2000);

    // Botón
    if ( (p.x >= 1500) && (p.x < 2500) && (p.y >= 1600) && (p.y < 2400) ) // Coordenadas del botón
    {
      if (state == 1) {
        state = 0;
      }
      else {
        state = 1;
      }
    }
    if (state == 0)
    {
      verd();
      // Verde

    }
    else
    {
      red();
      // Rojo

    }

  }


  // Wait for a short time before repeating the loop

  delay(500);
}


void dibujaGraficos()
{
  tft.fillScreen(ILI9341_BLACK);
  yield();

  // Titulo
  tft.fillRect(0, 0, 319, 60, ILI9341_BLUE);
  tft.drawRect(0, 0, 319, 60, ILI9341_WHITE);

  tft.setCursor(4, 30);
  tft.setTextColor(ILI9341_WHITE);  tft.setTextSize(3);
  tft.println("REFLOW SOLDERING");

  //Subtitulo
  tft.fillRect(0, 280, 180, 240, ILI9341_BLUE);
  tft.drawRect(0, 280, 180, 240, ILI9341_WHITE);

  tft.setCursor(4, 200);
  tft.setTextColor(ILI9341_WHITE);  tft.setTextSize(2);
  tft.println("Soldering");
  tft.setCursor(4, 220);
  tft.println(" Machine");

  // Bat level
  char texto[15];
  sprintf(texto, "juni.ana.jordi");
  tft.setCursor(320 - (strlen(texto) * 6), 5);
  tft.setTextColor(ILI9341_WHITE);  tft.setTextSize(1);
  tft.println(texto);

  // Separdor horizontal, vertical
  //tft.drawLine(0, 180, 240, 180, ILI9341_WHITE);
  //tft.drawLine(120, 60, 120, 280, ILI9341_WHITE);
  //TEMPERATURA
  tft.setCursor(180, 125);
  tft.setTextColor(ILI9341_WHITE);  tft.setTextSize(2);
  tft.println("TEMPERATURE");
  // ---------------
  // Botón
  tft.fillRect(20, 100, 110, 70, ILI9341_GREEN);   // P

  tft.setCursor(50, 125);
  tft.setTextColor(ILI9341_BLACK);  tft.setTextSize(2);
  tft.println("START");
  delay(1000);

}
void verd() {
  tft.fillRect(210, 80, 110, 40, ILI9341_BLACK);
  tft.fillRect(20, 100, 110, 70, ILI9341_GREEN);
  tft.setCursor(50, 125);
  tft.setTextColor(ILI9341_BLACK);  tft.setTextSize(2);
  tft.println("START");
  currentTime = 1;
}
void red() {
  tft.fillRect(20, 100, 110, 70, ILI9341_RED);
  tft.setCursor(50, 125);
  tft.setTextColor(ILI9341_WHITE);  tft.setTextSize(2);
  tft.println("STOP");
}
